from typing import Generator

from openai import Stream
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

CONTENT = "content"

T_IterItemsStream = Generator[ChatCompletionChunk, None, None]


def iter_answer_partial(stream: Stream) -> Generator[str, None, None]:
    iter_items: T_IterItemsStream = (item for item in stream)
    for item in iter_items:
        if hasattr(item.choices[0].delta, CONTENT):
            answer_partial = item.choices[0].delta.content
            if not answer_partial:
                continue
            yield answer_partial

def answer_from_stream(stream: Stream) -> str:
    answer = ""
    for answer_partial in iter_answer_partial(stream=stream):
        answer += answer_partial
    return answer



from typing import List, Dict
from abc import ABC

from openai import OpenAI, Stream
from openai.resources.chat.completions import Completions
from openai.resources.embeddings import Embeddings
from openai.resources.audio.transcriptions import Transcriptions

from oaikit.utils import answer_from_stream

#DEFAULT_OPENAI_API_BASE = "https://api.openai.com/v1"
#openai.api_base = os.getenv(OPENAI_API_BASE, DEFAULT_OPENAI_API_BASE)
DEFAULT_MODEL = "gpt-4o"

class BaseOAI(ABC):
    def __init__(self, *, api_key: str):
        self._client = OpenAI(api_key=api_key)

    @property
    def client(self) -> OpenAI:
        return self._client
    
    @property
    def completions(self) -> Completions:
        return self.client.chat.completions

    @property
    def embeddings(self) -> Embeddings:
        return self.client.embeddings

    @property
    def transcriptions(self) -> Transcriptions:
        return self.client.audio.transcriptions

    def get_stream(self, *, messages: List[Dict[str, str]], model: str) -> Stream:
        return self.completions.create(model=model, messages=messages, stream=True)

    def answer_from_messages(self, *, messages: List[Dict[str, str]], model: str = DEFAULT_MODEL) -> str:
        stream = self.get_stream(messages=messages, model=model)
        answer = answer_from_stream(stream=stream)
        return answer


class OAI(BaseOAI):
    def __init__(self, *, api_key: str):
        super().__init__(api_key=api_key)



from typing import List, Type, TypeVar

from pydantic import BaseModel

from oaikit.role import OAIRole, SYSTEM, USER, ASSISTANT
from oaikit.content.text import ContentText
from oaikit.content.image import ContentImage, ImageURL

__all__ = ["OAIMsg", "ContentText", "ContentImage", "ImageURL", "OAIRole"]

T_OAIMsg = TypeVar("T_OAIMsg", bound="OAIMsg")
T_Content = str | List[ContentText | ContentImage]

class OAIMsg(BaseModel):
    role: OAIRole
    content: T_Content
    #name: str ?

    @property
    def is_system(self) -> bool:
        return self.role == SYSTEM
    
    @property
    def is_user(self) -> bool:
        return self.role == USER
    
    @property
    def is_assistant(self) -> bool:
        return self.role == ASSISTANT

    @classmethod
    def system(cls: Type[T_OAIMsg], *, content: T_Content) -> Type[T_OAIMsg]:
        return cls(role=SYSTEM, content=content)
    
    @classmethod
    def user(cls: Type[T_OAIMsg], *, content: T_Content) -> Type[T_OAIMsg]:
        return cls(role=USER, content=content)

    @classmethod
    def assistant(cls: Type[T_OAIMsg], *, content: T_Content) -> Type[T_OAIMsg]:
        return cls(role=ASSISTANT, content=content)



from typing import Literal, Type, TypeVar
from pathlib import Path
import base64

from pydantic import BaseModel

__all__ = ["ImageURL", "ContentImage"]

IMAGE_PNG = "image/png"     # Literal?
DEFAULT_DETAIL = "auto"
DetailImageURL = Literal["auto", "low", "high"]


def encode_img(path_img: Path) -> str:
    with open(path_img, "rb") as f:
        img_b64 = base64.b64encode(f.read()).decode("utf-8")
    return img_b64


class ImageURL(BaseModel):
    """ https://platform.openai.com/docs/guides/vision/low-or-high-fidelity-image-understanding."""
    url: str
    detail: DetailImageURL = DEFAULT_DETAIL


T_ContentImage = TypeVar("T_ContentImage", bound="ContentImage")
class ContentImage(BaseModel):
    image_url: ImageURL
    type: Literal["image_url"] = "image_url"

    @classmethod
    def from_path(cls: Type[T_ContentImage], *, path_img: Path, detail: DetailImageURL = DEFAULT_DETAIL) -> Type[T_ContentImage]:
        """ Abre y codifica la imagen a base64.
        - FIXME: Creo que el image/png, es el content-type o algo similar.
        """
        img_b64 = encode_img(path_img=path_img)
        return cls.from_b64(img_b64=img_b64, detail=detail)
    
    @classmethod
    def from_b64(cls: Type[T_ContentImage], *, img_b64: str, detail: DetailImageURL = DEFAULT_DETAIL) -> Type[T_ContentImage]:
        url = f"data:{IMAGE_PNG};base64,{img_b64}"
        return cls.from_url(url=url, detail=detail)

    @classmethod
    def from_url(cls: Type[T_ContentImage], *, url: str, detail: DetailImageURL = DEFAULT_DETAIL) -> Type[T_ContentImage]:
        return cls(image_url=ImageURL(url=url, detail=detail))



from pathlib import Path
from setuptools import find_packages, setup

setup(
    name = "oaikit",
    version = "0.0.9",
    author = "Alejo Prieto Dávalos",
    author_email = "alejoprietodavalos@gmail.com",
    packages = find_packages(),
    description = "Python wrapper for OpenAI API.",
    long_description = Path("README.md").read_text(),
    long_description_content_type = "text/markdown",
    url = "https://pypi.org/project/oaikit/",
    project_urls = {
        "Source": "https://github.com/AlejoPrietoDavalos/oaikit/"
    },
    python_requires = ">=3.11",
    install_requires = [
        "pydantic>=2.8",
        "openai>=1.35.13"
    ],
    include_package_data = True
)



from enum import Enum

__all__ = ["OAIModels"]

class OAIModels(Enum):
    """ `OAIModels.GPT_4o`"""
    GPT_4o = "gpt-4o"
    GPT_4 = "gpt-4"
    GPT_3_5_TURBO = "gpt-3.5-turbo"

    def __str__(self):
        return self.value
